<?php
/**
 * DBERP 进销存系统
 *
 * ==========================================================================
 * @link      http://www.dberp.net/
 * @copyright 北京珑大钜商科技有限公司，并保留所有权利。
 * @license   http://www.dberp.net/license.html License
 * ==========================================================================
 *
 * @author    静静的风 <baron@loongdom.cn>
 *
 */

namespace Finance\Repository;

use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\Query\Expr\Join;
use Doctrine\ORM\QueryBuilder;
use Finance\Entity\Receivable;
use Sales\Entity\SalesOrder;

class ReceivableRepository extends EntityRepository
{
    /**
     * 应收账款列表sql
     * @param array $search
     * @return \Doctrine\ORM\Query
     */
    public function findReceivablesList(array $search = []): \Doctrine\ORM\Query
    {
        $query = $this->getEntityManager()->createQueryBuilder()
            ->select('r')
            ->from(Receivable::class, 'r')
            //->where('r.receivableCode = :code')
            //->setParameter('code', 'receivable')
            ->orderBy('r.receivableId', 'DESC');

        $query = $this->querySearchData($search, $query);

        return $query->getQuery();
    }

    /**
     * 计算合计金额
     * @param array $search
     * @return float|int|mixed[]|string
     */
    public function findReceivablesTotalAmount(array $search = [])
    {
        $query = $this->getEntityManager()->createQueryBuilder();
        $query->select('SUM(r.receivableAmount) as totalAmount', 'SUM(r.finishAmount) as finishAmount')
            ->from(Receivable::class, 'r');

        $query = $this->querySearchData($search, $query);

        return $query->getQuery()->getArrayResult();
    }

    /**
     * 获取列表信息
     * @param $receivableIdArray
     * @param $customerId
     * @return float|int|mixed|string
     */
    public function findReceivableIdList($receivableIdArray, $customerId)
    {
        $query = $this->getEntityManager()->createQueryBuilder();
        $query->select('r')
            ->from(Receivable::class, 'r')
            ->where($query->expr()->in('r.receivableId', $receivableIdArray))
            ->andWhere($query->expr()->eq('r.customerId', ':customerId'))->setParameter('customerId', $customerId);

        return $query->getQuery()->getResult();
    }

    /**
     * ajax应收账款列表
     * @param $customerId
     * @return float|int|mixed|string
     */
    public function ajaxFindAllReceivable($customerId)
    {
        $query = $this->getEntityManager()->createQueryBuilder();
        $query->select('r')
            ->from(Receivable::class, 'r')
            ->where('r.customerId = :customerId')->setParameter('customerId', $customerId)
            ->andWhere('r.salesInvoiceId = 0')
            ->orderBy('r.receivableId', 'DESC');

        return $query->getQuery()->getResult();
    }

    /**
     * 对检索信息进行处理
     * @param $search
     * @param QueryBuilder $query
     * @return QueryBuilder
     */
    private function querySearchData($search, QueryBuilder $queryBuilder): QueryBuilder
    {
        if(isset($search['sales_start_amount']) && $search['sales_start_amount'] > 0)   $queryBuilder->andWhere($queryBuilder->expr()->gte('r.receivableAmount', $search['sales_start_amount']));
        if(isset($search['sales_end_amount']) && $search['sales_end_amount'] > 0)       $queryBuilder->andWhere($queryBuilder->expr()->lte('r.receivableAmount', $search['sales_end_amount']));
        if(isset($search['start_amount']) && $search['start_amount'] > 0)               $queryBuilder->andWhere($queryBuilder->expr()->gte('r.finishAmount', $search['start_amount']));
        if(isset($search['end_amount']) && $search['end_amount'] > 0)                   $queryBuilder->andWhere($queryBuilder->expr()->lte('r.finishAmount', $search['end_amount']));
        if(!empty($search['sales_order_sn']))                                           $queryBuilder->andWhere($queryBuilder->expr()->like('r.salesOrderSn', "'%".$search['sales_order_sn']."%'"));
        if(!empty($search['customer_name']))                                            $queryBuilder->andWhere($queryBuilder->expr()->like('r.customerName', "'%".$search['customer_name']."%'"));
        if(!empty($search['receivable_code']))                                          $queryBuilder->andWhere($queryBuilder->expr()->eq('r.receivableCode', ':receivableCode'))->setParameter('receivableCode', $search['receivable_code']);
        if(!empty($search['start_date']))                                               $queryBuilder->andWhere($queryBuilder->expr()->gte('r.addTime', ':startTime'))->setParameter('startTime', strtotime($search['start_date']));
        if(!empty($search['end_date']))                                                 $queryBuilder->andWhere($queryBuilder->expr()->lte('r.addTime', ':endTime'))->setParameter('endTime', strtotime($search['end_date']));

        return $queryBuilder;
    }
}